package app;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import java.sql.*;

/**
 * @author CloudDuggu
 * Provide derby database services.
 */

@RestController
@SpringBootApplication
public class DerbyController {

    static String url;
    private Connection connect = null;

    public static void main(String... args) {
        url = args[0];
        SpringApplication.run(DerbyController.class, args);
    }

    @PostMapping({"/openDatabase"})
    public String openDatabase() {
        try {
            System.out.println("# Database Url: "+url);
            Class.forName("org.apache.derby.jdbc.ClientDriver").newInstance();
            connect = DriverManager.getConnection(url+";create=true"); // url -> jdbc:derby://localhost:1527/EMPDB
            if (connect != null)
                return "Database Connection Successfully.";
            else
                return "Error: Server Exception.";
        } catch (Exception ex) {
            return "Exception: " + ex.getMessage();
        }
    }

    @PostMapping({"/closeDatabase"})
    public String closeDatabase() {
        try {
            if (connect != null)
                connect.close();
            return "Close Database Connection Successfully.";
        } catch (Exception ex) {
            return "Exception: " + ex.getMessage();
        }
    }

    @PostMapping({"/executeQuery"})
    public String executeQuery(@RequestParam String data) {
        System.out.println("# Query: "+data);
        try {
            if (connect == null)
                openDatabase();

            StringBuffer sb = new StringBuffer("");
//            split data string and execute one query at a time.
            for(String query : data.split(";")) {
                if(!query.isEmpty())
                    sb.append(execute(query));
            }
            return sb.toString();
        } catch (Exception ex) {
            return "Exception: " + ex.getMessage();
        }
    }

    String execute(String data) {
        try {
            Statement statement = connect.createStatement();
            StringBuffer sb = new StringBuffer("");
            if(data.toUpperCase().contains("SELECT")) {

//                execute select query into database
                ResultSet resultSet = statement.executeQuery(data);
                ResultSetMetaData rsmd = resultSet.getMetaData();
                int columnsNumber = rsmd.getColumnCount();

//                fetch table columns names
                sb.append("<table class='table table-bordered table-hover table-responsive'><thead><tr class='table-info'>");
                for (int i = 1; i <= columnsNumber; i++) {
                    sb.append("<td>" + rsmd.getColumnName(i) + "</td>");
                }
                sb.append("</tr><tbody>");

//                fetch table row data
                while (resultSet.next()) {
                    sb.append("<tr>");
                    for (int i = 1; i <= columnsNumber; i++) {
                        sb.append("<td>" + resultSet.getString(i) + "</td>");
                    }
                    sb.append("</tr>");
                }
                sb.append("</tbody></table>");
            } else {
//                execute queries into database
                int updatedRows = statement.executeUpdate(data);
                sb.append("Query Execute Successfully. UpdatedRows: "+updatedRows+"<br/>");
            }
            return sb.toString();
        } catch (Exception ex) {
            return "Exception: " + ex.getMessage();
        }
    }

}